// ======================================================================
//
// MenuHorizontalSliderBar.cpp
// ~Chuck Karpiak
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MenuHorizontalSliderBar.h"

#include "Bitmap.h"
#include "Iff.h"
#include "Mouse.h"

// ======================================================================

const Tag TAG_BITS = TAG(B,I,T,S);
const Tag TAG_HSBE = TAG(H,S,B,E);

// ======================================================================

MenuHorizontalSliderBar::MenuHorizontalSliderBar(MenuElement *newParent, int maxWidth, int maxHeight, Iff *iff)
: MenuElement(newParent, maxWidth, maxHeight, NULL)
{
	percentage        = 0.0f;
	xOffset           = 0;
	eventNumber       = 0;
	sliderBarWidth    = 0;
	sliderBarXHotSpot = 0;
	bitmap            = NULL;
	pressingLeftMouse = false;

	load(iff);
}

// ----------------------------------------------------------------------

MenuHorizontalSliderBar::~MenuHorizontalSliderBar(void)
{
}

// ----------------------------------------------------------------------

void MenuHorizontalSliderBar::load(Iff *iff)
{
	char buffer[128];
	int  y;

	if (iff)
	{
		MenuElement::load(iff);

		iff->seekChunk(TAG_BITS);
		iff->read_string(buffer, sizeof(buffer));
		if ((bitmap = findSharedBitmap(buffer)) == NULL)
			Fatal("MenuHorizontalSliderBar::load !findSharedBitmap '%s'", buffer);
		if (bitmap->frames() < 2)
			Fatal("MenuHorizontalSliderBar::load shared bitmap '%s' only contains %d frames (need >= 2)", buffer, bitmap->frames());
		iff->exitChunk();

		// record off the slider bar's height for use in computing xOffset
		sliderBarWidth   = bitmap->width(1);
		bitmap->getHotspot(&sliderBarXHotSpot, &y);

		iff->seekChunk(TAG_HSBE);
		eventNumber = iff->read_int32();
		iff->exitChunk();
	}

	// initialize the xOffset
	clampXOffset();
}

// ----------------------------------------------------------------------
// hover event messages occur only once for each 'start' and 'stop'
IoResult MenuHorizontalSliderBar::event(IoEventStruct *event)
{
	static IoWin* old_capturer = NULL;
	Rect          r;

	r = menuRect;
	if (pressingLeftMouse)
		getAbsoluteRect(&r);

	if (r.isInInclusive(event->mouseX, event->mouseY))
	{
		// start dragging?
		if (event->type == IoEventMousePress && event->button == 0)
		{
			pressingLeftMouse = true;
			old_capturer = IoWinManager::capture (this);
			return IoResultBlock;
		}

		// end dragging?
		if (event->type == IoEventMouseRelease && event->button == 0)
		{
			pressingLeftMouse = false;
			// stop capturing
			IoWinManager::release (this, old_capturer);
			return IoResultBlock;
		}
	}
	else
		if (event->type == IoEventMouseRelease && event->button == 0)
		{
			pressingLeftMouse = false;
			// stop capturing
			IoWinManager::release (this, old_capturer);
			return IoResultBlock;
		}

	// are we currently dragging?
	if (pressingLeftMouse && event->type == IoEventMouseSetCursor)
	{
		calculatePercentage(menuRect.x0 + event->mouseX - r.x0);
		calculateXOffset();
		MenuElement::menuEvent(eventNumber, this);
		return IoResultBlock;
	}

	if (pressingLeftMouse)
		return IoResultBlock;

	return MenuElement::event(event);
}

// ----------------------------------------------------------------------

void MenuHorizontalSliderBar::menuDraw(int x, int y) const
{
	MenuElement::menuDraw(x, y);

	bitmap->draw(x + menuRect.x0, y + menuRect.y0, 0);
	bitmap->draw(x + menuRect.x0 + xOffset, y + menuRect.y0, 1);

//	GL::line(x + menuRect.x0 - 10, y + menuRect.y0 + sliderBarYHotSpot, x + menuRect.x1 + 10, y + menuRect.y0 + sliderBarYHotSpot, GL::green());
//	GL::line(x + menuRect.x0 - 10, y + menuRect.y1 - (sliderBarWidth - sliderBarYHotSpot), x + menuRect.x1 + 10, y + menuRect.y1 - (sliderBarWidth - sliderBarYHotSpot), GL::green());
//
//	GL::line(x + menuRect.x0 - 10, y + menuRect.y0+ yOffset, x + menuRect.x1 + 10, y + menuRect.y0 + yOffset, GL::white());
}

// ----------------------------------------------------------------------

void MenuHorizontalSliderBar::calculatePercentage(int mouseX)
{
	percentage =  static_cast<real>(mouseX - menuRect.x0 - sliderBarXHotSpot);
	percentage /= menuRect.x1 - menuRect.x0 - (sliderBarWidth);

//DPRINT_LOG(("percentage before = %f\n", percentage));

	// clamp it between 0.0f and 1.0f
	if (percentage < CONST_REAL(0))
		percentage = CONST_REAL(0);
	if (percentage > CONST_REAL(1))
		percentage = CONST_REAL(1);
}

// ----------------------------------------------------------------------
// assumes that curValue has been updated
void MenuHorizontalSliderBar::calculateXOffset(void)
{
	// set the slider drawing offset
	xOffset = static_cast<int>(percentage * (menuRect.x1 - menuRect.x0 - (sliderBarWidth))) + sliderBarXHotSpot;
	clampXOffset();
}

// ----------------------------------------------------------------------
// clamp down the xOffset to include the slider bar's height
inline void MenuHorizontalSliderBar::clampXOffset(void)
{
	// clamp down the xOffset to include the slider bar's height
	if (xOffset < sliderBarXHotSpot)
		xOffset = sliderBarXHotSpot;
	if (xOffset > (menuRect.x1 - (sliderBarWidth - sliderBarXHotSpot - 1)) - menuRect.x0)
		xOffset = (menuRect.x1 - (sliderBarWidth - sliderBarXHotSpot - 1)) - menuRect.x0;
}

// ----------------------------------------------------------------------

void MenuHorizontalSliderBar::setPercentage(real newPercentage)
{
	percentage = newPercentage;
	if (percentage < CONST_REAL(0))
		percentage = CONST_REAL(0);
	if (percentage > CONST_REAL(1))
		percentage = CONST_REAL(1);

	calculateXOffset();
}

// ======================================================================
